"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const child_process_1 = require("child_process");
const path = require("path");
const run_1 = require("../base/run");
const conf_1 = require("../services/conf");
const docker_1 = require("../services/docker");
const validate_1 = require("../services/validate");
const { resolve } = path;
const { RdTypes, cwd, rdcConfName } = conf_1.default;
class Run extends run_1.default {
    constructor() {
        super(...arguments);
        this.rebuild = false;
    }
    get from() {
        if (conf_1.default.rdType === RdTypes.Application) {
            const { container } = conf_1.default.getAppConf();
            return container.name;
        }
        if (conf_1.default.rdType === RdTypes.Container) {
            const rdcConfPath = resolve(cwd, rdcConfName);
            const rdcConf = require(rdcConfPath);
            return rdcConf.extends || rdcConf.nodeVersion || 'node:latest';
        }
    }
    get workDir() {
        const { dockerWorkDirRoot } = conf_1.default;
        let name;
        if (conf_1.default.rdType === RdTypes.Application) {
            const { container } = conf_1.default.getAppConf();
            name = container.name.split(':')[0];
        }
        if (conf_1.default.rdType === RdTypes.Container) {
            const rdcConfPath = resolve(cwd, rdcConfName);
            const rdcConf = require(rdcConfPath);
            const { tag } = rdcConf.docker;
            name = tag.split(':')[0];
        }
        return `${dockerWorkDirRoot}/${name}`;
    }
    get tag() {
        if (conf_1.default.rdType === RdTypes.Container) {
            const rdcConf = require(`${conf_1.default.cwd}/${conf_1.default.rdcConfName}`);
            return rdcConf.docker.tag || null;
        }
        if (conf_1.default.rdType === RdTypes.Application) {
            // name app image with project dir name
            return path.basename(cwd);
        }
    }
    get ports() {
        if (conf_1.default.rdType === RdTypes.Application) {
            const { docker } = conf_1.default.getAppConf();
            return docker.ports || [];
        }
        if (conf_1.default.rdType === RdTypes.Container) {
            const rdcConf = require(`${conf_1.default.cwd}/${conf_1.default.rdcConfName}`);
            return rdcConf.docker.ports || [];
        }
    }
    async preInit() {
        await super.preInit();
        const { flags } = this.parse(Run);
        this.rebuild = flags.rebuild;
        await docker_1.default.checkEnv(true);
        if (conf_1.default.rdType === RdTypes.Container) {
            await validate_1.validateRdc();
        }
        if (conf_1.default.rdType === RdTypes.Application) {
            await validate_1.validateRda();
        }
    }
    async preRun() {
        await docker_1.default.genDockerFile(this.workDir, this.from, conf_1.default.localCacheDir, conf_1.default.rdType === RdTypes.Application);
        await docker_1.default.genDockerCompose(this.workDir, this.cmd, this.ports, this.watch, `dev-${this.tag}`, conf_1.default.localCacheDir, conf_1.default.rdType === RdTypes.Application);
        await docker_1.default.genPkgJson();
    }
    async run() {
        // not using docker-compose cuz .dockerignore in sub dir is not working,
        // build with docker-compose is slow if node_modules exists
        await docker_1.default.build(`dev-${this.tag}`, conf_1.default.localCacheDir, this.rebuild);
        let args = ['run', '--rm', '--service-ports', 'rde', 'rde', 'docker:run', this.cmd];
        if (this.watch) {
            args.push('--watch');
        }
        if (this.extras) {
            args = args.concat(['-e', this.extras]);
        }
        let child = null;
        process.on('SIGINT', () => {
            child.kill();
        });
        child = child_process_1.spawn('docker-compose', args, {
            cwd: conf_1.default.localCacheDir,
            stdio: 'inherit',
        });
        child.on('close', code => {
            if (code !== 0) {
                process.exit(code);
            }
        });
    }
}
Run.description = 'run scripts provided by container';
Run.examples = [
    '$ rde run <cmd>',
];
Run.flags = Object.assign({}, run_1.default.flags, { rebuild: command_1.flags.boolean({
        char: 'r',
        description: 'rebuild image before run',
    }), staged: command_1.flags.boolean({
        char: 's',
        description: 'lint staged',
    }) });
Run.args = [{
        name: 'cmd',
        required: false,
        description: 'scripts provided by container',
    }];
exports.default = Run;
