"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const extend = require("deep-extend");
const fs = require("fs");
const path = require("path");
const writePackage = require("write-pkg");
const util_1 = require("../util");
const cache_1 = require("./cache");
const conf_1 = require("./conf");
const logger_1 = require("./logger");
const render_1 = require("./render");
const { resolve } = path;
const { RdTypes } = conf_1.default;
class Docker {
    async checkEnv(timesaving = false) {
        // whether is running or not
        if (timesaving) {
            await util_1.default.asyncExec('docker info');
        }
        else {
            await util_1.default.asyncExec('docker -v & docker-compose -v & docker info');
        }
    }
    async pull(image) {
        await util_1.default.asyncExec(`docker pull ${image}`);
    }
    async copy(image, mappings) {
        if (await this.containerExist('rde-dummy')) {
            await util_1.default.asyncExec('docker container rm /rde-dummy');
        }
        await util_1.default.asyncExec(`docker create -it --name rde-dummy ${image} bash`);
        for (const mapping of mappings) {
            await util_1.default.asyncExec(`docker cp rde-dummy:${mapping.from} ${mapping.to}`);
        }
        await util_1.default.asyncExec('docker rm -fv rde-dummy');
    }
    async imageExist(name) {
        try {
            await util_1.default.asyncExec(`docker inspect --type=image ${name}`);
            return true;
        }
        catch (e) {
            if (e) { }
            return false;
        }
    }
    async containerExist(name) {
        try {
            await util_1.default.asyncExec(`docker ps -a | grep ${name}`);
            return true;
        }
        catch (e) {
            if (e) { }
            return false;
        }
    }
    async build(tag, cwd, noCache = false, context = '.') {
        if (!noCache && await this.imageExist(tag)) {
            return;
        }
        let args = ['build', '-t', tag, '-f', 'Dockerfile', context];
        if (noCache) {
            args.splice(1, 0, '--no-cache');
        }
        await util_1.default.asyncSpawn('docker', args, {
            cwd,
        });
        await util_1.default.asyncExec('rm .dockerignore');
    }
    async tag(oldtag, newtag) {
        await util_1.default.asyncSpawn('docker', ['tag', oldtag, newtag]);
    }
    async push(name) {
        await util_1.default.asyncSpawn('docker', ['push', name]);
    }
    async genDockerFile(workDir, from, dir, isApp) {
        // gen dockerfile
        await render_1.default.renderTo('docker/.dockerignore', {}, '.dockerignore', {
            overwrite: true,
        });
        await render_1.default.renderTo('docker/Dockerfile', {
            from,
            workDir,
            dockerWorkDirRoot: conf_1.default.dockerWorkDirRoot,
            isApp,
        }, `${dir}/Dockerfile`, {
            overwrite: true,
        });
    }
    async genDockerFile4Publish(workDir, from, dir) {
        // gen dockerfile
        await render_1.default.renderTo('docker/.dockerignore', {}, '.dockerignore', {
            overwrite: true,
        });
        await render_1.default.renderTo('docker/Dockerfile.publish', {
            from,
            workDir,
        }, `${dir}/Dockerfile`, {
            overwrite: true,
        });
    }
    async genDockerCompose(workDir, cmd, ports, watch, tag, dir, isApp = false) {
        if (!tag) {
            const name = path.basename(conf_1.default.cwd);
            tag = `${name}:latest`;
        }
        await render_1.default.renderTo('docker/docker-compose', {
            workDir,
            cmd,
            ports,
            watch,
            tag,
            isApp,
        }, `${dir}/docker-compose.yml`, {
            overwrite: true,
        });
    }
    /**
     * rde will regenerate package.json under such condition:
     * 1. no cache info found
     * 2. rdc version changed if rdType is application
     * 3. any package.json under template dir has changed if rdType is container
     * 4. extends attr in rdc.config.js has changed if rdType is container
     */
    async genPkgJson() {
        const { rdType, cwd, localCacheDir, rdcConfName, } = conf_1.default;
        const isApplication = rdType === RdTypes.Application;
        let rdcName;
        if (isApplication) {
            const rdaConf = conf_1.default.getAppConf();
            rdcName = rdaConf.container.name || '';
        }
        const isContainer = rdType === RdTypes.Container;
        let pkgJsonLMTs;
        let extendsName;
        let pkgJson = null;
        if (isContainer) {
            // last-modified-time
            const pkgPath = resolve(cwd, 'template/package.json');
            pkgJsonLMTs = fs.statSync(pkgPath).mtimeMs;
            pkgJson = require(pkgPath);
            const rdcConfPath = resolve(cwd, rdcConfName);
            const rdcConf = require(rdcConfPath);
            extendsName = rdcConf.extends;
        }
        if (!cache_1.default.exist ||
            (isApplication && rdcName !== cache_1.default.get('rda.container')) ||
            (isContainer && pkgJsonLMTs !== cache_1.default.get('rdc.pkg.lmts')) ||
            (isContainer && extendsName && extendsName !== cache_1.default.get('rdc.extends'))) {
            const image = isApplication ? rdcName : extendsName;
            const resultPkgJson = await this.getPkgJson(image, pkgJson);
            await writePackage(`${localCacheDir}`, resultPkgJson);
            if (isApplication) {
                cache_1.default.set('rda.container', rdcName);
            }
            if (isContainer) {
                cache_1.default.set('rdc.pkg.lmts', pkgJsonLMTs);
                extendsName && cache_1.default.set('rdc.extends', extendsName);
            }
        }
    }
    async getPkgJson(image, result = {}) {
        if (!image) {
            return result;
        }
        const { dockerWorkDirRoot, localCacheDir, rdcConfName, cwd, } = conf_1.default;
        const name = image.split(':')[0];
        const srcDir = `${dockerWorkDirRoot}/${name}`;
        const destPkgPath = `${localCacheDir}/package.json`;
        const destRdcConfPath = `${localCacheDir}/${rdcConfName}`;
        await this.copy(image, [{
                from: `${srcDir}/template/package.json`,
                to: destPkgPath,
            }, {
                from: `${srcDir}/${rdcConfName}`,
                to: destRdcConfPath,
            }]);
        const rdcConf = require(resolve(cwd, destRdcConfPath));
        let pkgJson = {};
        if (fs.existsSync(destPkgPath)) {
            pkgJson = require(resolve(cwd, destPkgPath));
        }
        pkgJson = extend(pkgJson, result);
        await util_1.default.asyncExec(`rm ${destPkgPath} ${destRdcConfPath}`);
        if (!rdcConf.extends) {
            return pkgJson;
        }
        await this.getPkgJson(rdcConf.extends, pkgJson);
    }
}
tslib_1.__decorate([
    logger_1.default('Checking local environment...')
], Docker.prototype, "checkEnv", null);
tslib_1.__decorate([
    logger_1.default('Pulling image from docker hub...')
], Docker.prototype, "pull", null);
tslib_1.__decorate([
    logger_1.default('Copying files from image...')
], Docker.prototype, "copy", null);
tslib_1.__decorate([
    logger_1.default('Checking image existance...')
], Docker.prototype, "imageExist", null);
tslib_1.__decorate([
    logger_1.default('Checking container existance...')
], Docker.prototype, "containerExist", null);
tslib_1.__decorate([
    logger_1.default('Building docker image...')
], Docker.prototype, "build", null);
tslib_1.__decorate([
    logger_1.default('Pushing image to docker hub...')
], Docker.prototype, "push", null);
tslib_1.__decorate([
    logger_1.default('Generating docker files...')
], Docker.prototype, "genDockerFile", null);
tslib_1.__decorate([
    logger_1.default('Generating Dockerfile...')
], Docker.prototype, "genDockerFile4Publish", null);
tslib_1.__decorate([
    logger_1.default('Generating docker-compose.yml...')
], Docker.prototype, "genDockerCompose", null);
tslib_1.__decorate([
    logger_1.default('Generating package.json from rdc chain...')
], Docker.prototype, "genPkgJson", null);
exports.default = new Docker();
